//
//  DataSourceController.h
//  NewDSCTest
//
//  Created by bagheera on 7/11/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <pthread.h>
//#import "NSMutableArrayDSCAdditions.h"
//#import "NSArrayDSCAdditions.h"
//#import "DataSourceProtocol.h"
//#import "DSCGlobals.h"
//#import "GlobalKeys.h"

#import "DataSourceCall.h";
#import "DataSource.h";
#import "DataSourceHolder.h";
#import "DSFloatInfrequent.h"
#import "DSFloatFrequent.h"
#import "DSFloatFreqNonRepeat.h"
#import "DSFloatInfreqRepeat.h"
#import "DSColorHolder.h"
#import "DSPointHolder.h"
#import "VVDSBuiltIn.h"






/*

			A GENERAL DESCRIPTION:

	'DataSourceHolder' is the replacement for categories of data sources.  The concept of concrete,
1-level categories has been done away with, because it was inherently limiting the flexibility: not all
data sources (or data source categories, for that matter) should be treated equally.  Holders are told
to process while the DSC is iterating, and subclasses of DataSourceHolder may define their own proc.
For example, there'd be a MIDI ctrl holder, a MIDI noteon/off holder, and potentially even holders for
various plugins (which would make things very, very efficient).
	'DataSourceHolder' instances may be nested!
	The whole point of 'DataSource' is to provide a holder to store a value.  Different subclasses of
'DataSource' hold different types of data source (floats, BOOl, points, RGB values, etc.).
	'DataSourceHolder' is a subclass of 'DataSource'.  think of it like this: 'DataSourceHolder' is a
'DataSource' whose value is a collection of other 'DataSource's.  Some 'DataSource' subclasses (like
RGB values, for example) have components (R, G, and B) as *well* as a value (RGB).  'DataSource-
Holder' classes consist, at their barest level, of a dict.  Each element in the dict is a 'DataSource'
(or, of course, another 'DataSourceHolder').

	'DataSource' objects are ONLY RETAINED in their 'DataSourceHolder'.  When they are removed
from the holder, they are autoreleased.  DataSources should ONLY BE REMOVED USIN THE PROVIDED
METHODS.


*/

@interface DataSourceController : NSObject {
	NSMutableDictionary		*elementDict;
	pthread_rwlock_t		elementLock;
	int					visibleDepth;	//	max depth, but only accounts for items visible in a contextual menu
	
	NSTimer				*threadTimer;
	//	the run loop that the thread spawns & runs in.  if a thread calls the proc and it's run loop doesn't match this, exit the thread immediately!
	NSRunLoop			*threadRunLoop;
	NSAutoreleasePool		*threadPool;
	int					runLoopCount;
	
	pthread_rwlock_t		externalSynchLock;
	NSMutableArray		*externalSynchArray;
	
	pthread_rwlock_t		midiLock;
	NSMutableArray		*midiMessagesQueue;
	
	IBOutlet id				sliderManager;
	IBOutlet id				buttonManager;
	IBOutlet id				appController;
	id					midiManager;
	id					oscManager;
	IBOutlet id				coordinator;
	
	VVDSBuiltIn			*builtInDataSources;
	
	BOOL					hardwareLearnMode;
	
	NSMutableArray		*selectedAltUIItemsArray;
	pthread_rwlock_t		altUIItemsLock;
}

+ (long) DSCTime;

- (void) launchTheThread;	//	DO NOT CALL THIS METHOD
- (void) threadLaunch:(id)sender;	//	DO NOT CALL THIS METHOD
- (void) firstThreadLaunch;
- (void) stopTheThread;

- (void) threadProc:(NSTimer *)t;

- (DataSource *) dataSourceForKeyArray:(NSArray *)a;
- (DataSource *) dataSourceForValidMutableKeyArray:(NSMutableArray *)a;
- (BOOL) setDataSource:(DataSource *)ds forKeyArray:(NSArray *)a;
- (BOOL) changeNameFrom:(NSArray *)o to:(NSArray *)n;

- (BOOL) isThisDetectedSrcArraySafe:(NSArray *)a;

- (void) checkForBrokenSlidersAtKeyArray:(NSArray *)a;	//	called by DS'es when their parent source is set- used to check to see if UI items need to be re-synched

- (void) safelyRemoveObjectForKey:(NSString *)k;		//	DOES NOT PREP FOR DELETION!  WARNING!
- (void) safelyInsertObject:(id)o forKey:(NSString *)k fromDSC:(id)dsc;

- (BOOL) subscribeObject:(id)o selector:(SEL)s toFetchSourceArray:(NSArray *)a;
- (void) unsubscribeObject:(id)o fetchingFromSourceArray:(NSArray *)a;

- (void) somethingNeedsToFindMe:(NSNotification *)note;

- (void) addToExternalSynchArray:(id)n;
- (void) removeFromExternalSynchArray:(id)n;


- (void) recalculateDepth;	//	called internally when i need to re-calculate the depth of everything
- (void) setVisibleDepth:(int)n;
- (int) visibleDepth;
- (NSMenu *) getDataSourcesMenu;
- (NSMenu *) getDataSourceMenuWithDepth:(int)d;
- (NSMenu *) getColorDataSourcesMenu;
- (NSMenu *) getPointDataSourcesMenu;
- (NSMutableArray *) extractMenuItemsMatchingMask:(long)mask toLevel:(int)level carryOverToLevel:(int)reset;
- (NSMutableArray *) extractMenuItemsToLevel:(int)level carryOverToLevel:(int)reset;
- (NSMutableArray *) getDataSourceMenuItemArray;

- (void) midiMsgReceived:(id)msg;
- (void) oscDictReceived:(NSDictionary *)d;
- (void) keyDown:(NSEvent *)event;
- (void) programChangeReceived;

- (void) deselectEverything;

- (void) selectJustThisSlider:(id)n;
- (void) selectThisSliderToo:(id)n;
- (void) deselectThisSlider:(id)n;
- (void) deselectAllSliders;

- (void) selectThisButton:(id)n;
- (void) deselectThisButton:(id)n;
- (void) deselectAllButtons;

- (void) selectThisAltUIItem:(id)n;
- (void) deselectThisAltUIItem:(id)n;
- (void) deselectAllAltUIItems;

- (BOOL) hardwareLearnMode;
- (void) setHardwareLearnMode:(BOOL)n;

- (id) midiManager;
- (id) oscManager;



@end

long			DSCTime;

