//
//  VVAudioDeviceController.h
//  VVSliders
//
//  Created by David Lublin on 5/20/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

//	this is a combination of the old VVAudioDevice and VVAudioInput objects (mostly VVAudioDevice)
//	there is one major difference though.. no audio processing is done within this object
//	rather it's IOProc function just passes it's ABL off to any subscribed pluggins


#import <Cocoa/Cocoa.h>
#import <CoreAudio/CoreAudio.h>
#import <CoreAudio/AudioHardware.h>
#import <CoreAudio/CoreAudioTypes.h>
#import <AudioToolbox/AudioToolbox.h>
#import <IOKit/audio/IOAudioTypes.h>
#import "VVAudioPlugin.h"
@class VVAudioPlugin;

//	this is a struct for keeping track of subscribed pluggins (linked list)
//	this is done in C so I can do it in my IOProc FAST

@interface VVAudioDeviceController : NSObject {
	//	
	BOOL							isRunning;
	BOOL							inputsRunning;
	BOOL							outputsRunning;
	
	BOOL							deleted;
	
	NSMutableArray					*outputsArray;
	NSMutableArray					*inputsArray;
	
	//	device info
	AudioDeviceID					deviceID;
	NSString						*deviceName;
	
	AudioStreamBasicDescription		deviceFormat;
	AudioStreamBasicDescription		realInputDescription;
	AudioStreamBasicDescription		inputDescription;
	AudioStreamBasicDescription		realOutputDescription;
	AudioStreamBasicDescription		outputDescription;
	
	//BOOL							deviceIsInput;
	NSMutableArray					*inputBuffersArray;
	NSMutableArray					*outputBuffersArray;
	
	AudioBufferList						*fakeInputBuffer;
	AudioBufferList						*fakeOutputBuffer;
	
	int								inputChannelCount;
	int								outputChannelCount;
	UInt32							audioBufferSize;
	UInt32							deviceBufferSize;
	UInt32							deviceSafetyOffset;
	Float64							deviceSampleRate;
	UInt32							outputBufferSize;
	
	UInt32							inputFrameCount;
	UInt32							outputFrameCount;
	
	pthread_mutex_t					callbackLock;
}

+ (id) createWithDeviceId:(AudioDeviceID)newID;
- (id) initWithDeviceID:(AudioDeviceID)newID;
- (void) prepareToQuit;
- (void) prepareToBeDeleted;
- (void) subscribeObject:(VVAudioPlugin *)newSubscriber;
- (void) subscribeObject:(VVAudioPlugin *)newSubscriber isInput:(BOOL)inputFlag;
- (void) deleteObject:(VVAudioPlugin *)subscriber;
- (void) unsubscribeObject:(VVAudioPlugin *)subscriber;
- (void) unsubscribeObject:(VVAudioPlugin *)subscriber isInput:(BOOL)inputFlag;

//	note that this method should only be called from within the VVAudioDeviceController
- (void) setIsRunning:(BOOL)newState;

- (AudioDeviceID)deviceID;
- (NSString *)deviceName;
- (UInt32)deviceBufferSize;
- (UInt32)outputBufferSize;
- (Float64)deviceSampleRate;
- (int)inputChannelCount;
- (int)outputChannelCount;
- (AudioBufferList *)fakeInputBuffer;
- (AudioBufferList *)fakeOutputBuffer;
- (AudioStreamBasicDescription)inputDescription;
- (AudioStreamBasicDescription)outputDescription;
- (AudioStreamBasicDescription)deviceFormat;
- (pthread_mutex_t) callbackLock;
- (NSArray *)inputBuffersArray;
- (NSArray *)outputBuffersArray;
- (UInt32) audioBufferSize;
@end
