//
//  VVButtonCell.h
//  VDMX
//
//  Created by bagheera on 5/19/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "GlobalKeys.h"
//#import <pthread.h>
//#import "NSMutableArrayDSCAdditions.h"
#import "VVMIDIMsg.h"
#import "DataSourceController.h"
#import "VVButtonManager.h"
@class VVButtonManager;
//@class DataSource;
//#import "VVButton.h"
//@class VVButton;
//#import "TheApplication.h"


@interface VVButtonCell : NSCell <NSCopying, VVSnapshotProtocol> {
	BOOL					deleted;
	
	VVButton				*controlView;
	id					target;
	SEL					action;
	id					delegate;
	
	NSString				*title;			//	title when not-down
	NSString				*title2;			//	title when down
	NSString				*mixedString;		//	the text that is displayed when the button is mixed
	int					buttonValue;
	BOOL					isDown;			//	whether or not the mouse button is down
	NSTimer				*toggleRedrawTimer;	//	timer used to draw a flash when button is toggled via a shortcut of some sort
	int					modifiersOnClick;
	
	BOOL					selected;
	VVButtonManager		*buttonManager;
	DataSourceController	*dataSourceController;
	BOOL					showShortcut;
	
	BOOL					syncFlag;
	NSArray				*syncArray;
	int					triggerMode;		//	if NSOffState, i'm in straight-sync mode.  if NSOnState, use threshold-based triggering.
	BOOL					detectingMIDI;
	BOOL					detectingOSC;
	BOOL					detectingKey;
	BOOL					detectingButton;
	BOOL					syncDeletedUnderMe;	//	whether or not my current data source was deleted under me
	BOOL					hardwareLearnMode;	//	whether or not i'm in hardware-learn mode
	float					threshVal;
	BOOL					triggerOnRise;
	BOOL					triggerOnFall;
	BOOL					preventClickSelect;	//	for buttons in inspectors; prevent selection unless shift-clicked
	BOOL					triggerOnMain;		//	for buttons that change the synch mode of other sliders or buttons (buttons that trigger re-entrant threads)
	
	BOOL					preventSyncToggle;	//	i don't want to make toggles-for-toggles-for-toggles; if YES, then i can't setSyncToggleState to YES
	id					syncToggle;		//	the sync toggle button
	NSDictionary			*syncToggleDict;	//	dict of the settings for the sync toggle
	
	NSMutableArray		*lockedButtonArray;	//	buttons can be locked, unifying their states/status/actions.  this is the array of buttons that are synched to me.
	BOOL					hasLockedButtons;	//	whether or not a button has other buttons which are locked to it
	VVButtonCell			*lockedButtonSrc;	//	if i'm locked to another slider, this is the slider i'm locked to
	BOOL					lockedFlag;		//	whether or not i'm locked to another button
	pthread_mutex_t		lockedButtonLock;	//	if i tell the button i'm locked to to do something, and it tells me again, that's a feedback loop.  the lock prevents this.
	
	
	int					buttonType;		//	click or toggle; enum in VVButton.h
	float					lineWidth;			//	width of the stroke around the button
	float					radius;			//	radius of the button's rounded edges
	
	/*
	NSColor				*bgColor;			//	bg color when not-down
	NSColor				*bgColor2;		//	bg color when in transition
	NSColor				*bgColor3;		//	bg color when down
	NSColor				*strokeColor;		//	stroke color when not-down
	NSColor				*strokeColor2;		//	stroke color when in transition
	NSColor				*strokeColor3;		//	stroke color when down
	NSColor				*txtColor;		//	text color when not-down
	NSColor				*txtColor2;		//	text color when down
	*/
	//BOOL					useCustomColors;
	
	NSImage				*imgTiles;	//	a picture with "tiles" of the images to use for display.  L->R, it's up, transition, down.
	NSSize				imgTileSize;
	
	//NSString				*fontName;		//	name of the font for the titles
	//int					fontSize;			//	size of the font for the titles
	//NSDictionary			*fontAttributes;
	//NSDictionary			*fontAttributes2;
	//NSDictionary			*hardwareLearnFontAttribs;
	//NSDictionary			*hardwareLearnFontAttribs2;
}


//	init stuff
- (NSString *) description;
+ (id) createWithTitle:(NSString *)t synchedTo:(NSArray *)s;
- (id) copyWithZone:(NSZone *)zone;
- (void) encodeWithCoder:(NSCoder *)coder;
- (id) initWithCoder:(NSCoder *)coder;
- (void) prepareToBeDeleted;

//	preset save/restore
- (NSMutableDictionary *) createSnapshot;
- (void) loadSnapshot:(NSDictionary *)fxDict;
- (void) loadSnapshot:(NSDictionary *)fxDict andOutput:(BOOL)o;

//	misc
- (void) buttonManagerCheck;
- (void) hardwareLearnModeUpdated:(id)m;
- (void) showShortcutsNotification:(NSNotification *)note;
- (void) select;
- (void) deselect;
- (void) toggle;
- (void) bang;

//	button locking/unification
- (void) lockThisButtonToMe:(VVButtonCell *)lockToMe;
- (void) unlockMe;
- (NSMutableArray *) lockedButtonArray;
- (BOOL) hasLockedButtons;
- (VVButtonCell *) lockedButtonSrc;
- (void) setLockedButtonSrc:(VVButtonCell *)n;
- (BOOL) lockedFlag;
- (void) setLockedFlag:(BOOL)n;
- (BOOL) okayToTriggerLockedButtons;

//	DSC-related stuff
- (BOOL) useThisSource:(NSArray *)a;
- (void) dataSourceUpdated:(DataSource *)theSource;
- (int) dataSourceType;	//	if there's a valid DS at my syncArray, returns it's type- if there's no DS, returns VVDSNone.  returns the type even if not synced!
- (BOOL) validDataSourceValAvailable;	//	if there's a valid DS at my syncArray, and it's either a freq. DS or an infreq. DS that has been properly set, this returns YES
- (void) currentDataSourceDeletedUnderMe;
- (void) wasThisDataSourceDeletedOutFromUnderMe:(NSArray *)a;
- (void) dataSourceNameChanged:(DataSource *)ds;
- (void) setDataSourceWithVal:(DataSource *)source;
- (BOOL) syncFlag;
- (void) setSyncFlag:(BOOL)n;

//	methods for interacting with the per-button sync toggle
- (BOOL) preventSyncToggle;
- (void) setPreventSyncToggle:(BOOL)n;
- (id) syncToggle;
- (NSDictionary *) syncToggleDict;
- (void) setSyncToggleDict:(NSDictionary *)n;
- (void) setSyncToggleState:(BOOL)n;
- (void) syncToggleTriggered:(id)sender;

//	drawing stuff
- (void) useThisImageTile:(NSImage *)n withTileSize:(NSSize)s;
//- (void) updateColorsFromButtonManager;
- (void) drawWithFrame:(NSRect)cellFrame inView:(VVButton *)theView;
//- (NSBezierPath *) roundRectPathInRect:(NSRect)rectFrame withRadius:(float)newRadius;
- (NSDictionary *) getRelevantColorDict;
- (void) forceRedraw;

//	mouse interaction
- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(VVButton *)theView untilMouseUp:(BOOL)untilMouseUp;
- (BOOL) startTrackingAt:(NSPoint)startPoint inView:(VVButton *)theView;
- (BOOL) continueTracking:(NSPoint)lastPoint at:(NSPoint)currentPoint inView:(VVButton *)theView;
- (void) stopTracking:(NSPoint)lastPoint at:(NSPoint)stopPoint inView:(VVButton *)theView mouseIsUp:(BOOL)flag;
- (BOOL) prefersTrackingUntilMouseUp;

//	detect stuff
- (void) startMIDIDetect;
- (void) startOSCDetect;
- (void) startKeyDetect;
- (void) startButtonDetect;
- (void) startAllDetect;
- (void) stopMIDIDetect;
- (void) stopOSCDetect;
- (void) stopKeyDetect;
- (void) stopButtonDetect;
- (void) stopAllDetect;
- (void) midiDetectReceivedNotification:(NSNotification *)note;
- (void) oscDetectReceivedNotification:(NSNotification *)note;
- (void) keyDetectReceivedNotification:(NSNotification *)note;
- (void) buttonDetectReceivedNotification:(NSNotification *)note;

//	key-value
- (int) buttonType;
- (void) setButtonType:(int)n;
/*
- (float) lineWidth;
- (void) setLineWidth:(float)n;
- (float) radius;
- (void) setRadius:(float)n;
- (NSColor *) bgColor;
- (void) setBgColor:(NSColor *)n;
- (NSColor *) bgColor2;
- (void) setBgColor2:(NSColor *)n;
- (NSColor *) bgColor3;
- (void) setBgColor3:(NSColor *)n;
- (NSColor *) strokeColor;
- (void) setStrokeColor:(NSColor *)n;
- (NSColor *) strokeColor2;
- (void) setStrokeColor2:(NSColor *)n;
- (NSColor *) strokeColor3;
- (void) setStrokeColor3:(NSColor *)n;
- (NSColor *) txtColor;
- (void) setTxtColor:(NSColor *)n;
- (NSColor *) txtColor2;
- (void) setTxtColor2:(NSColor *)n;
*/
- (NSString *) title;
- (void) setTitle:(NSString *)n;
- (NSString *) title2;
- (void) setTitle2:(NSString *)n;
//- (NSString *) fontName;
//- (void) setFontName:(NSString *)n;
- (BOOL) isDown;
- (void) setIsDown:(BOOL)n;
- (NSTimer *) toggleRedrawTimer;
- (void) setToggleRedrawTimer:(NSTimer *)n;
//- (int) fontSize;
//- (void) setFontSize:(int)n;
- (VVButton *) controlView;
- (void) setControlView:(VVButton *)n;
- (id) target;
- (void) setTarget:(id)n;
- (SEL) action;
- (void) setAction:(SEL)n;
- (void) setBoolValue:(BOOL)n;
- (void) setBoolValue:(BOOL)n andOutput:(BOOL)o;
- (int) intValue;
- (void) setIntValue:(int)n;
- (void) setIntValue:(int)n andOutput:(BOOL)o;
- (float) floatValue;
- (void) setFloatValue:(float)n;
- (void) setFloatValue:(float)n andOutput:(BOOL)o;
- (NSArray *) syncArray;
- (BOOL) detectingMIDI;
- (BOOL) detectingOSC;
- (BOOL) detectingKey;
- (BOOL) detectingButton;
- (BOOL) preventClickSelect;
- (void) setPreventClickSelect:(BOOL)n;
- (BOOL) triggerOnMain;
- (void) setTriggerOnMain:(BOOL)n;
- (NSNumber *) triggerOnMainNumber;
- (void) setTriggerOnMainNumber:(NSNumber *)n;
- (id) delegate;
- (void) setDelegate:(id)newDelegate;

- (VVButtonManager *) buttonManager;
- (void) setButtonManager:(VVButtonManager *)n;
- (float) threshVal;
- (void) setThreshVal:(float)n;
- (BOOL) triggerOnRise;
- (void) setTriggerOnRise:(BOOL)n;
- (BOOL) triggerOnFall;
- (void) setTriggerOnFall:(BOOL)n;
- (int) triggerMode;
- (void) setTriggerMode:(int)n;


- (BOOL) deleted;
//- (BOOL) useCustomColors;
//- (void) setUseCustomColors:(BOOL)n;

- (int) modifiersOnClick;



/*
//	init stuff
- (NSString *) description;
+ (id) createWithTitle:(NSString *)t synchedTo:(NSArray *)s;

- (id) copyWithZone:(NSZone *)zone;
- (void) encodeWithCoder:(NSCoder *)coder;
- (id) initWithCoder:(NSCoder *)coder;

- (void) prepareToBeDeleted;

- (NSMutableDictionary *) createSnapshot;
- (void) loadSnapshot:(NSDictionary *)fxDict;




- (void) lockThisButtonToMe:(VVButtonCell *)lockToMe;
- (void) unlockMe;	//	unlocks anything- source button, non-source button....the works
- (NSMutableArray *) lockedButtonArray;
- (BOOL) hasLockedButtons;
- (VVButtonCell *) lockedButtonSrc;
- (void) setLockedButtonSrc:(VVButtonCell *)n;
- (BOOL) lockedFlag;
- (void) setLockedFlag:(BOOL)n;
- (BOOL) okayToTriggerLockedButtons;


- (void) useThisImageTile:(NSImage *)n withTileSize:(NSSize)s;
- (void) updateColorsFromButtonManager;
//	called by my cell when my bounds have changed & i need to re-render some paths
- (NSBezierPath *) roundRectPathInRect:(NSRect)rectFrame withRadius:(float)newRadius;


- (void) showShortcutsNotification:(NSNotification *)note;

- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(VVButton *)theView untilMouseUp:(BOOL)untilMouseUp;
- (BOOL) startTrackingAt:(NSPoint)startPoint inView:(VVButton *)theView;
- (BOOL) continueTracking:(NSPoint)lastPoint at:(NSPoint)currentPoint inView:(VVButton *)theView;
- (void) stopTracking:(NSPoint)lastPoint at:(NSPoint)stopPoint inView:(VVButton *)theView mouseIsUp:(BOOL)flag;
- (BOOL) prefersTrackingUntilMouseUp;

- (void) hardwareLearnModeUpdated:(id)m;
- (void) startMIDIDetect;
- (void) startKeyDetect;
- (void) startButtonDetect;
- (void) startAllDetect;
- (void) stopMIDIDetect;
- (void) stopKeyDetect;
- (void) stopButtonDetect;
- (void) stopAllDetect;
- (void) midiDetectReceivedNotification:(NSNotification *)note;
- (void) keyDetectReceivedNotification:(NSNotification *)note;
- (void) buttonDetectReceivedNotification:(NSNotification *)note;

- (BOOL) useThisSource:(NSArray *)a;
- (void) dataSourceUpdated:(DataSource *)theSource;
- (void) currentDataSourceDeletedUnderMe;
- (void) wasThisDataSourceDeletedOutFromUnderMe:(NSArray *)a;
- (void) dataSourceNameChanged:(DataSource *)ds;

- (int) buttonType;
- (void) setButtonType:(int)n;
- (float) lineWidth;
- (void) setLineWidth:(float)n;
- (float) radius;
- (void) setRadius:(float)n;
- (NSColor *) bgColor;
- (void) setBgColor:(NSColor *)n;
- (NSColor *) bgColor2;
- (void) setBgColor2:(NSColor *)n;
- (NSColor *) bgColor3;
- (void) setBgColor3:(NSColor *)n;
- (NSColor *) strokeColor;
- (void) setStrokeColor:(NSColor *)n;
- (NSColor *) strokeColor2;
- (void) setStrokeColor2:(NSColor *)n;
- (NSColor *) strokeColor3;
- (void) setStrokeColor3:(NSColor *)n;
- (NSColor *) txtColor;
- (void) setTxtColor:(NSColor *)n;
- (NSColor *) txtColor2;
- (void) setTxtColor2:(NSColor *)n;
- (NSString *) title;
- (void) setTitle:(NSString *)n;
- (NSString *) title2;
- (void) setTitle2:(NSString *)n;
- (NSString *) fontName;
- (void) setFontName:(NSString *)n;
- (BOOL) isDown;
- (void) setIsDown:(BOOL)n;
- (NSTimer *) toggleRedrawTimer;
- (void) setToggleRedrawTimer:(NSTimer *)n;
- (int) fontSize;
- (void) setFontSize:(int)n;

- (VVButton *) controlView;
- (void) setControlView:(VVButton *)n;
- (id) target;
- (void) setTarget:(id)n;
- (SEL) action;
- (void) setAction:(SEL)n;
- (int) intValue;
- (void) setIntValue:(int)n;
- (void) setIntValue:(int)n andOutput:(BOOL)o;
- (float) floatValue;
- (void) setFloatValue:(float)n;
- (NSArray *) syncArray;
- (BOOL) detectingMIDI;
- (BOOL) detectingKey;
- (BOOL) detectingButton;
- (BOOL) preventClickSelect;
- (void) setPreventClickSelect:(BOOL)n;
- (BOOL) triggerOnMain;
- (void) setTriggerOnMain:(BOOL)n;
- (id) delegate;
- (void) setDelegate:(id)newDelegate;


- (void) select;
- (void) deselect;
- (void) toggle;
- (void) bang;
- (VVButtonManager *) buttonManager;
- (void) setButtonManager:(VVButtonManager *)n;
- (BOOL) syncFlag;
- (void) setSyncFlag:(BOOL)n;
- (float) threshVal;
- (void) setThreshVal:(float)n;
- (BOOL) triggerOnRise;
- (void) setTriggerOnRise:(BOOL)n;
- (BOOL) triggerOnFall;
- (void) setTriggerOnFall:(BOOL)n;
- (BOOL) deleted;
- (BOOL) useCustomColors;
- (void) setUseCustomColors:(BOOL)n;



//	sometimes a button is a data source; this is the method that sets a data source with the button's value
- (void) setDataSourceWithVal:(DataSource *)source;
*/



@end
