//
//  VVSliderCell.h
//  VDMX
//
//  Created by bagheera on 5/12/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <OpenGL/CGLMacro.h>
#include <math.h>
#import "GlobalKeys.h"
#import "VVSliderCell.h"
#import "BehaviorChainManager.h"




@interface VVSliderCell : NSActionCell <NSCopying, VVSnapshotProtocol>	{
	id				sliderManager;
	id				dataSourceController;
	id				myTarget;
	SEL				myAction;
	//IBOutlet id			delegate;
	BOOL				deleted;	//	when something prepares to delete me, this gets set to YES and prevents data-source-related-things from happening to me
	BOOL				vertFlag;	//	if YES, the slider's vertical (the playhead runs top to bottom)
	
	NSString			*sliderName;
	float				lastInputValue;		//	when i set a slider's value, it sets this, then runs this through behavior chains, etc.
	float				value;			//	the actual slider value, after running through behavior chains
	float				lastReceivedDSValue;	//	the last value received from my DS; exists so soft takeover can work with infrequent data sources
	float				lastOutputValue;	//	the last value sent to my target.  this variable exists so i can avoid redraws if possible.
	NSMutableArray	*interestingPoints;	//	a sorted array of float-based NSNumbers with the vals (0.-1.) of the interesting points, which are snappable...
	NSMutableArray	*interestingPointRectArray;
	float				maxEnv;
	float				minEnv;
	float				maxRange;
	float				minRange;
	float				fillValue;	//	the slider's "bar" starts at this value (0.-1.).  default is 0.0
	long				uniqueID;
	BOOL				selected;
	BOOL				lockUIFlag;
	NSArray			*syncArray;	//	array of strings describing the key path to whatever i'm synched to
	BehaviorFXChain	*behaviorChain;
	NSString			*behaviorName;
	
	BOOL				softTakeoverFlag;	//	whether or not the slider is in the process of altering its value for a soft takeover
	BOOL				softTakeoverMIDICheck;	//	whether or not i should check for softTakeover on the next received MIDI val
	int				softTakeoverMode;		//	which mode is being used for soft takeover
	float				softTakeoverTolerance;	//	how close the DS val has to be to the last set value in order for takeover to happen
	float				softTakeoverSpeed;	//	how fast soft takeover moves
	long				lastTakeoverEvalTime;	//	last time (according to the DSC) when my takeover was evaluated
	
	NSMutableArray	*lockedSliderArray;	//	sliders can be locked together, unifying their states/status/actions.  this is the array of sliders that are synched to me.
	BOOL				hasLockedSliders;	//	whether or not a slider has other sliders which are locked to it
	VVSliderCell		*lockedSliderSrc;	//	if i'm locked to another slider, this is the slider i'm locked to
	BOOL				lockedFlag;		//	whether or not i'm locked to another slider
	pthread_mutex_t	lockedSliderLock;	//	if i tell the slider i'm locked to to do something, and it tells me again, that's a feedback loop.  the lock prevents this.
	
	float				clickValue;		//	the value of the slider when the last cell started tracking
	float				clickMaxEnv;		//	the value of maxEnv when the last cell started tracking
	float				clickMinEnv;		//	the value of minEnv when the last cell started tracking
	BOOL				mouseDown;		//	whether or not the mouse is down (duh)
	NSPoint			localClickLoc;		//	the location of the click in the cell when it started tracking
	int				clickedIPIndex;		//	the index of the interesting point i clicked on (if i clicked on one)
	int				clickModifiers;		//	the modifiers (shift, option, etc.) of the click in the cell that started tracking
	int				changeMask;		//	mask defined in GlobalKeys.h; indicates values that are changing during dragging
	int				jumpMask;		//	mask defined in GlobalKeys.h; indicates values that are being jumped during dragging

	BOOL				syncFlag;			//	whether or not i'm synched to something
	BOOL				syncDeletedUnderMe;	//	whether or not my current data source was deleted under me
	BOOL				behaviorFlag;		//	whether or not i'm using a behavior
	BOOL				hardwareLearnMode;	//	whether or not i'm in hardware-learn mode
	id				syncToggle;
	id				behavToggle;
	NSDictionary		*syncToggleDict;	//	dict of the settings for the sync toggle
	NSDictionary		*behavToggleDict;	//	ditto, but for the behavior toggle
	/*
	BOOL				presetsFlag;			//	whether or not i've got presets
	NSMutableArray	*presetDictArray;		//	array of dicts which contain slider snapshots
	NSMutableArray	*presetButtonArray;	//	array of buttons for presets
	id				presetRWButton;		//	the button which toggles between read & write mode for the presets
	*/
	VVSafeString		*sliderNameText;
	VVSafeString		*envText;
	VVSafeString		*valueText;
	VVSafeString		*syncText;
	
	NSColor			*bgColor;	//	nil, or the background color to use for filling the slider before drawing inside it
	NSImage			*lockIcon;
	
	int				valueTextRedrawCount;
	
	NSRect			controlFrame;
	NSRect			absControlFrame;	//	coords, expressed as absolute to the display
	NSRect			leftEnvHandle;
	NSRect			rightEnvHandle;
	NSRect			leftEnv;
	NSRect			rightEnv;
	NSRect			sliderHead;
	NSRect			sliderBG;
	
	NSBezierPath		*highlightPath;
	NSBezierPath		*shadowPath;
}


//	VVSliderCell used to be VVSliderControl- these are convenience methods, so when i
//	call NSControl-ish methods on myself (like [self bounds] and [self setFrame:blah]), they "just work"
- (NSRect) bounds;
- (NSRect) frame;
- (void) setNeedsDisplay:(BOOL)n;
- (void) keyDown:(NSEvent *)event;
- (void) keyUp:(NSEvent *)event;

- (id) init;
- (id) copyWithZone:(NSZone *)zone;
- (void) encodeWithCoder:(NSCoder *)coder;
- (id) initWithCoder:(NSCoder *)coder;

//	this method deletes the slider; it MUST be called so the slider's retain count is decremented in the manager
- (void) prepareToBeDeleted;
- (NSString *) description;

- (void) hardwareLearnModeUpdated:(id)m;
//- (void) drawRect:(NSRect)drawMe;
- (void) drawWithFrame:(NSRect)cellFrame inView:(id)theView;

//	misc
- (NSDictionary *) getRelevantColorDict;	//	gets the relevant color dict from the VVSliderManager's 'sliderColorDict'
- (NSDictionary *) hardCodedColorDict;
//- (BOOL) acceptsFirstMouse:(NSEvent *)theEvent;//	this method over-rides the superclasses default behavior of the first click in a non-key window bringing the window to front
- (id) target;
- (void) setTarget:(id)newTarget;
- (SEL) action;
- (void) setAction:(SEL)newAction;
//- (void) rightMouseDown:(NSEvent *)theEvent;		//	the slider's cell doesn't 'do' right-click (which is totally lame)
//- (void) rightMouseDownTimer:(NSTimer *)timer;
- (void) setFrame:(NSRect)f;
- (BOOL) isOpaque;
- (void) forceRedraw;

//	methods for updating the NSRects used to draw and track mouse interaction
- (void) updateMinEnvRects;
- (void) updateMaxEnvRects;
- (void) updateSliderValueRects;
- (void) populateBuildingRectsFromView:(id)controlView withFrame:(NSRect)cellFrame;

//	selection
- (void) selectJustMe;	//	called by the cell when i click (*not* shift-click) on a slider that's not selected
- (void) selectMeToo;	//	called by the cell when i shift-click on a slider that's not selected
- (void) deselectJustMe;	//	called by the cell when i shift-click on a slider that's selected
- (void) setSelected:(BOOL)newValue;
- (BOOL) selected;

//	the method that is called when another slider wants to be locked to this slider
- (void) lockThisSliderToMe:(VVSliderCell *)lockToMe;
- (void) unlockMe;	//	unlocks anything- source sliders, non-source sliders...the works
- (NSMutableArray *) lockedSliderArray;
- (BOOL) hasLockedSliders;
- (VVSliderCell *) lockedSliderSrc;
- (void) setLockedSliderSrc:(VVSliderCell *)n;
- (BOOL) lockedFlag;
- (void) setLockedFlag:(BOOL)n;
- (BOOL) okayToTriggerLockedSliders;
- (BOOL) lockUIFlag;
- (void) setLockUIFlag:(BOOL)n;

//	called by the VVSliderManager when the user changes the data source of the slider
- (BOOL) useThisSource:(NSArray *)a;
- (void) dataSourceNameChanged:(DataSource *)ds;
- (void) setSyncFlag:(BOOL)newValue;
- (void) startAllDetect;
- (void) startMIDIDetect;
- (void) startOSCDetect;
- (void) stopAllDetect;
- (void) stopMIDIDetect;
- (void) stopOSCDetect;
- (void) midiDetectReceivedNotification:(NSNotification *)note;
- (void) oscDetectReceivedNotification:(NSNotification *)note;
- (void) currentDataSourceDeletedUnderMe;	//	called by VVSliderManager when a data source i'm using is deleted
- (void) wasThisDataSourceDeletedOutFromUnderMe:(NSArray *)a;
- (void) dataSourceUpdated:(DSFloatFrequent *)theSource;
- (int) dataSourceType;	//	if there's a valid DS at my syncArray, returns it's type- if there's no DS, returns VVDSNone.  returns the type even if not synced!
- (BOOL) validDataSourceValAvailable;	//	if there's a valid DS at my syncArray, and it's either a freq. DS or an infreq. DS that has been properly set, this returns YES
- (void) useThisBehavior:(BehaviorFXChain *)newBehavior;
- (void) useBehaviorNamed:(NSString *)newBehaviorName;
- (void) setBehaviorFlag:(BOOL)newValue;
- (void) programChangeReceived;

//	methods for interacting with the per-slider sync toggle
- (id) syncToggle;
- (NSDictionary *) syncToggleDict;
- (void) setSyncToggleDict:(NSDictionary *)n;
- (void) setSyncToggleState:(BOOL)n;
- (void) syncToggleTriggered:(id)sender;

//	methods for interacting with the per-slider behavior toggle
- (id) behavToggle;
- (NSDictionary *) behavToggleDict;
- (void) setBehavToggleDict:(NSDictionary *)n;
- (void) setBehavToggleState:(BOOL)n;
- (void) behavToggleTriggered:(id)sender;

//	called by various behaviors; they alter the value/props of the slider WITHOUT USING BEHAVIORES OR CAUSING REDISPLAY
- (void) justSetValue:(float)newValue;
- (void) justSetMinenv:(float)newValue;
- (void) justSetMaxenv:(float)newValue;
//	sometimes a slider is a data source; this is the method that sets a data source with the slider's value
- (void) setDataSourceWithVal:(DSFloatFrequent *)source;
- (void) evaluateBehaviors;

//	interesting point methods
- (void) addInterestingPoint:(float)n;
- (void) addInterestingPointWithRangedValue:(float)rangedValue;
- (void) removeInterestingPoint:(float)n;
- (void) sortInterestingPoints;
- (NSNumber *) findRelevantInterestingPointForValue:(float)n;
- (NSNumber *) findInterestingPointClosestToValue:(float)n;
- (NSNumber *) findInterestingPointAtIndex:(int)n;
- (void) setValue:(NSNumber *)n forInterestingPointAtIndex:(int)i;
- (float) firstInterestingPointsValue;
- (float) lastInterestingPointsValue;
- (void) removeAllInterestingPoints;

//	data source and value-setting methods
- (void) bang;
- (void) setValue:(float)newValue;
- (void) setValue:(float)newValue andOutput:(BOOL)outputFlag;
- (void) setValueAndOutputIfDifferent:(float)newValue;
- (void) setMaxEnvNumber:(NSNumber *)n;
- (void) setMaxEnv:(float)newValue;
- (void) setMaxEnv:(float)newValue andOutput:(BOOL)output;
- (void) setMinEnvNumber:(NSNumber *)n;
- (void) setMinEnv:(float)newValue;
- (void) setMinEnv:(float)newValue andOutput:(BOOL)output;
- (void) setMaxRange:(float)newValue;
- (void) setMinRange:(float)newValue;
- (void) setValueWithRangedValue:(float)rangedValue;
- (void) setValueWithRangedValue:(float)rangedValue andOutput:(BOOL)outputFlag;
- (float) rangedMinEnv;
- (float) rangedMaxEnv;
- (float) rangedValue;
- (float) fillValue;
- (void) setFillValue:(float)n;
- (void) setFillValueWithRangedValue:(float)n;
- (void) jumpSliderBy:(float)jumpValue;
- (void) jumpSliderBy:(float)jumpValue andOutput:(BOOL)outputFlag;
- (void) jumpSliderByRangedValue:(float)jumpValue;
- (void) jumpSliderByRangedValue:(float)jumpValue andOutput:(BOOL)outputFlag;


//	misc. key-value
- (void) setSliderName:(NSString *)newName;
- (NSString *) sliderName;
- (void) setSliderManager:(id) newVal;
- (id) sliderManager;
- (float) lastInputValue;
- (float) value;
- (NSMutableArray *) interestingPoints;
- (float) maxEnv;
- (float) minEnv;
- (float) maxRange;
- (float) minRange;
- (void) setUniqueID:(long)newValue;
- (long) uniqueID;
- (BehaviorFXChain *) behaviorChain;
- (NSString *) behaviorName;
- (void) setClickValue:(float)newValue;
- (float) clickValue;
- (void) setClickMaxEnv:(float)newValue;
- (float) clickMaxEnv;
- (void) setClickMinEnv:(float)newValue;
- (float) clickMinEnv;
- (void) setLocalClickLoc:(NSPoint)newValue;
- (NSPoint) localClickLoc;
- (void) setClickedIPIndex:(int)n;
- (int) clickedIPIndex;
- (void) setClickModifiers:(int)newValue;
- (int) clickModifiers;
- (void) setChangeMask:(int)newValue;
- (int) changeMask;
- (void) setJumpMask:(int)newValue;
- (int) jumpMask;
- (BOOL) syncFlag;
- (NSArray *) syncArray;
- (BOOL) behaviorFlag;
- (void) setSoftTakeoverMode:(int)n;
- (void) setSoftTakeoverTolerance:(float)n;
- (void) setSoftTakeoverSpeed:(float)n;
//- (id) delegate;
//- (void) setDelegate:(id)n;
- (NSRect) absVisibleRect;
- (NSRect) controlFrame;
- (NSRect) absControlFrame;
- (NSRect) leftEnvHandle;		//	updated wherever minEnv is updated
- (NSRect) rightEnvHandle;	//	updated wherever maxEnv is updated
- (NSRect) leftEnv;			//	updated wherever minEnv is updated
- (NSRect) rightEnv;			//	updated wherever maxEnv is updated
- (NSRect) sliderHead;		//	updated wherever value is updated
- (NSRect) sliderBG;			//	updated wherever value is updated
- (BOOL) vertFlag;
- (void) setVertFlag:(BOOL)n;
- (BOOL) deleted;
- (void) setControlView:(id)n;

- (VVSafeString *) sliderNameText;
- (VVSafeString *) valueText;
- (VVSafeString *) syncText;


@end






/*
@interface VVSliderCell : NSActionCell {
	NSRect			controlFrame;
	NSRect			leftEnvHandle, rightEnvHandle;
	NSRect			leftEnv, rightEnv;
	NSRect			sliderHead;
	float				min, max, val;
	NSMutableArray	*interestingPointRectArray;
}

- (void) populateBuildingRectsFromView:(id)controlView withFrame:(NSRect)cellFrame;
- (NSRect) controlFrame;

@end
*/
