//
//  VVSliderControl.h
//  VDMX
//
//  Created by bagheera on 5/12/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <OpenGL/CGLMacro.h>
#import "GlobalKeys.h"
#import "VVSliderCell.h"
#import "BehaviorChainManager.h"



@interface VVSliderControl : NSControl <VVSnapshotProtocol> {
	BOOL					deleted;
	IBOutlet id				delegate;
	DataSourceController	*dataSourceController;
	//NSOpenGLContext		*context;
	//BOOL					contextNeedsRefresh;
}

+ (NSOpenGLPixelFormat *) defaultPixelFormat;
+ (id) createWithFrame:(NSRect)frame;

//	this method deletes the slider; it MUST be called so the slider's retain count is decremented in the manager
- (void) prepareToBeDeleted;
- (NSString *) description;

//- (void) hardwareLearnModeUpdated:(id)m;

//	misc
//- (NSDictionary *) getRelevantColorDict;	//	gets the relevant color dict from the VVSliderManager's 'sliderColorDict'
//- (NSDictionary *) hardCodedColorDict;
//- (BOOL) acceptsFirstMouse:(NSEvent *)theEvent;//	this method over-rides the superclasses default behavior of the first click in a non-key window bringing the window to front
//- (id) target;
- (void) setTarget:(id)newTarget;
//- (SEL) action;
- (void) setAction:(SEL)newAction;
- (void) rightMouseDown:(NSEvent *)theEvent;		//	the slider's cell doesn't 'do' right-click (which is totally lame)
- (void) rightMouseDownTimer:(NSTimer *)timer;
- (void) setFrame:(NSRect)f;
- (BOOL) isOpaque;
//- (void) forceRedraw;

//	methods for updating the NSRects used to draw and track mouse interaction
//- (void) updateMinEnvRects;
//- (void) updateMaxEnvRects;
//- (void) updateSliderValueRects;

//	selection
- (void) selectJustMe;	//	called by the cell when i click (*not* shift-click) on a slider that's not selected
- (void) selectMeToo;	//	called by the cell when i shift-click on a slider that's not selected
- (void) deselectJustMe;	//	called by the cell when i shift-click on a slider that's selected
- (void) setSelected:(BOOL)newValue;
- (BOOL) selected;

//	the method that is called when another slider wants to be locked to this slider
- (void) lockThisSliderToMe:(VVSliderControl *)lockToMe;
- (void) unlockMe;	//	unlocks anything- source sliders, non-source sliders...the works
//- (NSMutableArray *) lockedSliderArray;
//- (BOOL) hasLockedSliders;
//- (VVSliderControl *) lockedSliderSrc;
- (void) setLockedSliderSrc:(VVSliderControl *)lockToMe;
//- (BOOL) lockedFlag;
- (void) setLockedFlag:(BOOL)lockToMe;
//- (BOOL) okayToTriggerLockedSliders;
- (BOOL) lockUIFlag;
- (void) setLockUIFlag:(BOOL)n;

//	called by the VVSliderManager when the user changes the data source of the slider
- (BOOL) useThisSource:(NSArray *)a;
- (void) dataSourceNameChanged:(DataSource *)ds;
- (void) setSyncFlag:(BOOL)newValue;
- (void) startAllDetect;
- (void) startMIDIDetect;
- (void) startOSCDetect;
- (void) stopAllDetect;
- (void) stopMIDIDetect;
- (void) stopOSCDetect;
//- (void) midiDetectReceivedNotification:(NSNotification *)note;
//- (void) oscDetectReceivedNotification:(NSNotification *)note;
//- (void) currentDataSourceDeletedUnderMe;	//	called by VVSliderManager when a data source i'm using is deleted
//- (void) wasThisDataSourceDeletedOutFromUnderMe:(NSArray *)a;
//- (void) dataSourceUpdated:(DSFloatFrequent *)theSource;
- (int) dataSourceType;	//	if there's a valid DS at my syncArray, returns it's type- if there's no DS, returns VVDSNone.  returns the type even if not synced!
- (BOOL) validDataSourceValAvailable;
//- (void) useThisBehavior:(BehaviorFXChain *)newBehavior;
//- (void) useBehaviorNamed:(NSString *)newBehaviorName;
//- (void) setBehaviorFlag:(BOOL)newValue;

//	methods for interacting with the per-slider sync toggle
- (id) syncToggle;
- (NSDictionary *) syncToggleDict;
- (void) setSyncToggleDict:(NSDictionary *)n;
- (void) setSyncToggleState:(BOOL)n;
- (void) syncToggleTriggered:(id)sender;

//	methods for interacting with the per-slider behavior toggle
- (id) behavToggle;
- (NSDictionary *) behavToggleDict;
- (void) setBehavToggleDict:(NSDictionary *)n;
- (void) setBehavToggleState:(BOOL)n;
- (void) behavToggleTriggered:(id)sender;

//	called by various behaviors; they alter the value/props of the slider WITHOUT USING BEHAVIORES OR CAUSING REDISPLAY
- (void) justSetValue:(float)newValue;
- (void) justSetMinenv:(float)newValue;
- (void) justSetMaxenv:(float)newValue;
//	sometimes a slider is a data source; this is the method that sets a data source with the slider's value
- (void) setDataSourceWithVal:(DSFloatFrequent *)source;
- (void) evaluateBehaviors;

//	interesting point methods
- (void) addInterestingPoint:(float)n;
- (void) addInterestingPointWithRangedValue:(float)rangedValue;
- (void) removeInterestingPoint:(float)n;
//- (void) sortInterestingPoints;
- (NSNumber *) findRelevantInterestingPointForValue:(float)n;
- (NSNumber *) findInterestingPointClosestToValue:(float)n;
- (NSNumber *) findInterestingPointAtIndex:(int)n;
- (void) setValue:(NSNumber *)n forInterestingPointAtIndex:(int)i;
- (float) firstInterestingPointsValue;
- (float) lastInterestingPointsValue;
- (void) removeAllInterestingPoints;

//	data source and value-setting methods
- (void) bang;
- (void) setValue:(float)newValue;
- (void) setValue:(float)newValue andOutput:(BOOL)outputFlag;
- (void) setValueAndOutputIfDifferent:(float)newValue;
- (void) setMaxEnvNumber:(NSNumber *)n;
- (void) setMaxEnv:(float)newValue;
- (void) setMaxEnv:(float)newValue andOutput:(BOOL)output;
- (void) setMinEnvNumber:(NSNumber *)n;
- (void) setMinEnv:(float)newValue;
- (void) setMinEnv:(float)newValue andOutput:(BOOL)output;
- (void) setMaxRange:(float)newValue;
- (void) setMinRange:(float)newValue;
- (void) setValueWithRangedValue:(float)rangedValue;
- (void) setValueWithRangedValue:(float)rangedValue andOutput:(BOOL)outputFlag;
- (float) rangedMinEnv;
- (float) rangedMaxEnv;
- (float) rangedValue;
- (float) fillValue;
- (void) setFillValue:(float)n;
- (void) setFillValueWithRangedValue:(float)n;
- (void) jumpSliderBy:(float)jumpValue;
- (void) jumpSliderBy:(float)jumpValue andOutput:(BOOL)outputFlag;
- (void) jumpSliderByRangedValue:(float)jumpValue;
- (void) jumpSliderByRangedValue:(float)jumpValue andOutput:(BOOL)outputFlag;


//	misc. key-value
- (void) setSliderName:(NSString *)newName;
- (NSString *) sliderName;
- (void) setSliderManager:(id) newVal;
- (id) sliderManager;
- (float) lastInputValue;
- (float) value;
- (NSMutableArray *) interestingPoints;
- (float) maxEnv;
- (float) minEnv;
- (float) maxRange;
- (float) minRange;
- (void) setUniqueID:(long)newValue;
- (long) uniqueID;
- (BehaviorFXChain *) behaviorChain;
- (NSString *) behaviorName;
- (void) setClickValue:(float)newValue;
- (float) clickValue;
- (void) setClickMaxEnv:(float)newValue;
- (float) clickMaxEnv;
- (void) setClickMinEnv:(float)newValue;
- (float) clickMinEnv;
- (void) setLocalClickLoc:(NSPoint)newValue;
- (NSPoint) localClickLoc;
- (void) setClickedIPIndex:(int)n;
- (int) clickedIPIndex;
- (void) setClickModifiers:(int)newValue;
- (int) clickModifiers;
- (void) setChangeMask:(int)newValue;
- (int) changeMask;
- (void) setJumpMask:(int)newValue;
- (int) jumpMask;
- (BOOL) syncFlag;
- (NSArray *) syncArray;
- (BOOL) behaviorFlag;
- (void) setSoftTakeoverMode:(int)n;
- (void) setSoftTakeoverTolerance:(float)n;
- (void) setSoftTakeoverSpeed:(float)n;
- (id) delegate;
- (void) setDelegate:(id)n;
- (NSRect) absVisibleRect;
- (NSRect) controlFrame;
- (NSRect) absControlFrame;
- (NSRect) leftEnvHandle;		//	updated wherever minEnv is updated
- (NSRect) rightEnvHandle;	//	updated wherever maxEnv is updated
- (NSRect) leftEnv;			//	updated wherever minEnv is updated
- (NSRect) rightEnv;			//	updated wherever maxEnv is updated
- (NSRect) sliderHead;		//	updated wherever value is updated
- (NSRect) sliderBG;			//	updated wherever value is updated
- (BOOL) vertFlag;
- (BOOL) deleted;
- (DataSourceController *) dataSourceController;
- (void) setDataSourceController:(DataSourceController *)n;
//- (NSOpenGLContext *) context;

- (VVSafeString *) sliderNameText;
- (VVSafeString *) valueText;
- (VVSafeString *) syncText;



@end
